// distribution boxbackup-0.11rc5 (svn version: 2579_2580M)
//  
// Copyright (c) 2003 - 2008
//      Ben Summers and contributors.  All rights reserved.
//  
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. All use of this software and associated advertising materials must 
//    display the following acknowledgment:
//        This product includes software developed by Ben Summers.
// 4. The names of the Authors may not be used to endorse or promote
//    products derived from this software without specific prior written
//    permission.
// 
// [Where legally impermissible the Authors do not disclaim liability for 
// direct physical injury or death caused solely by defects in the software 
// unless it is modified by a third party.]
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//  
//  
//  
// --------------------------------------------------------------------------
//
// File
//		Name:    HTTPQueryDecoder.cpp
//		Purpose: Utility class to decode HTTP query strings
//		Created: 26/3/04
//
// --------------------------------------------------------------------------

#include "Box.h"

#include <stdlib.h>

#include "HTTPQueryDecoder.h"

#include "MemLeakFindOn.h"


// --------------------------------------------------------------------------
//
// Function
//		Name:    HTTPQueryDecoder::HTTPQueryDecoder(
//			 HTTPRequest::Query_t &)
//		Purpose: Constructor. Pass in the query contents you want
//			 to decode the query string into.
//		Created: 26/3/04
//
// --------------------------------------------------------------------------
HTTPQueryDecoder::HTTPQueryDecoder(HTTPRequest::Query_t &rDecodeInto)
	: mrDecodeInto(rDecodeInto),
	  mInKey(true),
	  mEscapedState(0)
{
	// Insert the terminator for escaped characters
	mEscaped[2] = '\0';
}


// --------------------------------------------------------------------------
//
// Function
//		Name:    HTTPQueryDecoder::~HTTPQueryDecoder()
//		Purpose: Destructor.
//		Created: 26/3/04
//
// --------------------------------------------------------------------------
HTTPQueryDecoder::~HTTPQueryDecoder()
{
}


// --------------------------------------------------------------------------
//
// Function
//		Name:    HTTPQueryDecoder::Decode(const char *, int)
//		Purpose: Decode a chunk of query string -- call several times with
//				 the bits as they are received, and then call Finish()
//		Created: 26/3/04
//
// --------------------------------------------------------------------------
void HTTPQueryDecoder::DecodeChunk(const char *pQueryString, int QueryStringSize)
{
	for(int l = 0; l < QueryStringSize; ++l)
	{
		char c = pQueryString[l];
		
		// BEFORE unescaping, check to see if we need to flip key / value
		if(mEscapedState == 0)
		{
			if(mInKey && c == '=')
			{
				// Set to store characters in the value
				mInKey = false;
				continue;
			}
			else if(!mInKey && c == '&')
			{
				// Need to store the current key/value pair
				mrDecodeInto.insert(HTTPRequest::QueryEn_t(mCurrentKey, mCurrentValue));
				// Blank the strings
				mCurrentKey.erase();
				mCurrentValue.erase();
			
				// Set to store characters in the key
				mInKey = true;
				continue;
			}
		}
		
		// Decode an escaped value?
		if(mEscapedState == 1)
		{
			// Waiting for char one of the escaped hex value
			mEscaped[0] = c;
			mEscapedState = 2;
			continue;
		}
		else if(mEscapedState == 2)
		{
			// Escaped value, decode it
			mEscaped[1] = c;	// str terminated in constructor
			mEscapedState = 0;	// stop being in escaped mode
			long ch = ::strtol(mEscaped, NULL, 16);
			if(ch <= 0 || ch > 255)
			{
				// Bad character, just ignore
				continue;
			}
			
			// Use this instead
			c = (char)ch;
		}		
		else if(c == '+')
		{
			c = ' ';
		}
		else if(c == '%')
		{
			mEscapedState = 1;
			continue;
		}

		// Store decoded value into the appropriate string
		if(mInKey)
		{
			mCurrentKey += c;
		}
		else
		{
			mCurrentValue += c;
		}
	}
	
	// Don't do anything here with left over values, DecodeChunk might be called
	// again. Let Finish() clean up.
}


// --------------------------------------------------------------------------
//
// Function
//		Name:    HTTPQueryDecoder::Finish()
//		Purpose: Finish the decoding. Necessary to get the last item!
//		Created: 26/3/04
//
// --------------------------------------------------------------------------
void HTTPQueryDecoder::Finish()
{
	// Insert any remaining value.
	if(!mCurrentKey.empty())
	{
		mrDecodeInto.insert(HTTPRequest::QueryEn_t(mCurrentKey, mCurrentValue));
		// Blank values, just in case
		mCurrentKey.erase();
		mCurrentValue.erase();
	}
}


